#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#	Name:	Fixed Area
#		By:	sdmitch
#Usage:	Dynamically change the dimensions of a rectangle while maintaining the original area.
# Note:	
#			
# Date:	Sep 2011
#------------------------------------------------------------------------------------------------
require 'sketchup'

filename=File.basename(__FILE__)
if !file_loaded?(filename)
	UI.menu("Plugins").add_item("Fixed Area") { Sketchup.active_model.select_tool Offset_Edge.new }
end
file_loaded(filename)

class Offset_Edge

	def initialize
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@Mod = Sketchup.active_model
		@Ent = @Mod.active_entities
		@Sel = @Mod.selection
		@Vue = @Mod.active_view
		self.reset
	end

	def reset
		@dy = 0.0
		@state = 0
		@best = nil
		@ip1.clear
		@Sel.clear
		@drawn = false
	end

	def onMouseMove(flags, x, y, view)
		self.set_current_point(x, y, view)
		ph = view.pick_helper
		ph.do_pick(x, y)
		@best = ph.best_picked if ph.best_picked != @best
		Sketchup.set_status_text("Select a Face or Right Click to exit",SB_PROMPT) if @state==0
		Sketchup.set_status_text("Select an Edge or Right Click for new Face",SB_PROMPT) if @state==1
		Sketchup.set_status_text("Point or Keyin to define new dimension, Left Click to set or Right Click for new Edge",SB_PROMPT) if @state==2
		view.refresh
	end

	def set_current_point(x, y, view)
		if( !@ip.pick(view, x, y, @ip1) )
			return false
		end
		@ip1.copy! @ip
		view.tooltip = @ip.tooltip     
	end

	def onLButtonDown(flags, x, y, view)
		self.set_current_point(x, y, view)
		if @state == 0 then
			if (@best.is_a?(Sketchup::Face)) then
				@face=@best;@pts=[];@vec=[]
				@edges=@face.outer_loop.edges
				if @edges.length != 4 || @edges[0].length != @edges[2].length || @edges[1].length != @edges[3].length then
					UI.messagebox "The selected face is not a rectangle"
				else
					@edges.reverse! if @edges[0].reversed_in?(@face)
					for i in 0...@edges.length
						@pts[i],@vec[i]=@edges[i].line
					end
					@area=@face.area
					@Sel.add @face; 
					norm=@face.normal; norm.reverse! if norm.z < 0.0
					cam_tar=@face.bounds.center
					cam_eye=cam_tar.offset(norm,Math.sqrt(@area)*3.0)
					cam=Sketchup::Camera.new
					cam.set(cam_eye,cam_tar,norm.axes[1])
					@Vue.camera=cam
					@Vue.refresh
					@state += 1
				end
			else
				UI.beep
			end
		elsif @state == 1 then
			if (@best.is_a?(Sketchup::Edge)) then
				@Sel.add @best
				@ndx=@edges.index(@best)
				@line=@edges[@ndx-2].line
				@state += 1
			else
				UI.beep
			end
		elsif @state == 2 then
			@Mod.start_operation "Fixed Area"
			pt0=@ip.position
			dy = @dy>0 ? @dy : pt0.distance_to_line(@line); dx=@area/dy
			p0=@pts[@ndx-2]
			p1=p0.offset(@vec[@ndx-2],dx)
			tr=Geom::Transformation.new(p1-@edges[@ndx-2].end.position)
			@Ent.transform_entities(tr,@edges[@ndx-2].end)
			p2=p1.offset(@vec[@ndx-1],dy)
			tr=Geom::Transformation.new(p2-@edges[@ndx-1].end.position)
			@Ent.transform_entities(tr,@edges[@ndx-1].end)
			p3=p2.offset(@vec[@ndx],dx)
			tr=Geom::Transformation.new(p3-@edges[@ndx].end.position)
			@Ent.transform_entities(tr,@edges[@ndx].end)
			@Mod.commit_operation
			self.reset
		end
		view.lock_inference
	end
	
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end
	
	def onRButtonDown(flags, x, y, view)
		@Mod.select_tool(nil) if @state==0
		@state-=1
	end

	def draw(view)
		@drawn = false
		# Show the current input point
		if( @ip.valid? && @ip.display? ) then
			@ip.draw(view)
			@drawn = true
		end
		if( @state == 2 )
			pt0=@ip.position
			dy = @dy>0 ? @dy : pt0.distance_to_line(@line); dx=@area/dy
			Sketchup.set_status_text("New Dimension:",SB_VCB_LABEL)
			Sketchup.set_status_text("#{dy.to_l}",SB_VCB_VALUE)
			p0=@pts[@ndx-2]
			p1=p0.offset(@vec[@ndx-2],dx)
			p2=p1.offset(@vec[@ndx-1],dy)
			p3=p2.offset(@vec[@ndx],dx)
			view.drawing_color = "magenta" # [255,0,255]
			view.line_width = 3 #if inference_locked
			view.draw(GL_LINE_LOOP, p0,p1,p2,p3)
			view.line_width = 1 #if inference_locked
			@drawn = true
		end
	end

	def onUserText(text, view)
		begin
			value = text.to_l
		rescue
			UI.beep
			value = nil
		end
		return if !value
		if value > 0.0
			@dy = value
			view.invalidate
		end
	end
	
end # of class OffsetEdge
